import json
import logging
import os


class Manifest:
    __manifest = None

    def __init__(self):
        self.__load_manifest()

    def __load_manifest(self):
        logging.info("Load manifest - manifest.json")
        cwd = os.getcwd()
        with open(f"{cwd}/manifest.json") as file:
            self.__manifest = json.load(file)
            logging.info(self.__manifest)

    def get(self, key):
        return self.__manifest[key]


manifest = Manifest()



from apispec import APISpec
from apispec.ext.marshmallow import MarshmallowPlugin
from flask import Flask
from flask_apispec import FlaskApiSpec
from flask_restful import Api

from openfabric_pysdk.json import PyJSON
from openfabric_pysdk.manifest import manifest


class OpenfabricService:
    __app = None
    __api = None
    __docs = None

    input_type: PyJSON = None
    output_type: PyJSON = None
    execution_function = None

    def __init__(self, app: Flask):
        self.__app = app
        self.__install_specs()
        self.__register = dict()
        self.__api = Api(app)
        self.__docs = FlaskApiSpec(app)

    def __install_specs(self):
        specs = {
            'APISPEC_SPEC': APISpec(
                title=manifest.get('name'),
                version=manifest.get('version'),
                plugins=[MarshmallowPlugin()],
                openapi_version='2.0.0',
                info=dict(
                    termsOfService='https://openfabric.ai/terms/',
                    contact=dict(name=manifest.get('organization'), url="https://openfabric.ai"),
                    description=manifest.get('description')),
            ),
            'APISPEC_SWAGGER_URL': '/spec/',  # URI to access API Doc JSON
            'APISPEC_SWAGGER_UI_URL': '/spec-ui/'  # URI to access UI of API Doc
        }
        self.__app.config.update(specs)

    def install(self, target, endpoint):
        self.__api.add_resource(target, endpoint)
        self.__docs.register(target)

    def run(self, debug, host):
        self.__app.run(debug=debug, host=host)



import time

from flask_apispec import marshal_with, doc, use_kwargs
from flask_apispec.views import MethodResource
from flask_restful import Resource
from marshmallow import Schema

from openfabric_pysdk.manifest import manifest
from openfabric_pysdk.schema import DefinitionSchema
from openfabric_pysdk.service import OpenfabricService
from openfabric_pysdk.util import Util


class OpenfabricRestApi(MethodResource, Resource):

    @doc(description="Input/Output definition", tags=[manifest.get("overview")])
    @marshal_with(DefinitionSchema)  # marshalling
    def get(self):
        return {
            'input': Util.get_concept_definition("input"),
            'output': Util.get_concept_definition("output")
        }

    @doc(description="Entrypoint", tags=[manifest.get("overview")])
    @use_kwargs(OpenfabricService.input_type.schema, location='json')
    @marshal_with(OpenfabricService.output_type.schema)  # marshalling
    def post(self, **kwargs) -> OpenfabricService.output_type:
        start = time.time()
        # Input Schema
        schema_clazz = Util.import_class(OpenfabricService.input_type.schema)
        json = schema_clazz(many=False).load(kwargs)

        # Input Object
        object_clazz = Util.import_class(OpenfabricService.input_type)
        instance = object_clazz(json)
        result = OpenfabricService.execution_function(instance)
        end = time.time()
        duration = round(end - start, 2)
        print(f"Processing time: {duration}s")

        return result



from pydoc import locate

from openfabric_pysdk.manifest import manifest


class Util:
    @staticmethod
    def get_concept_definition(selection):
        concept_id = manifest.get(selection).split('/')[1]
        schema = manifest.get(selection).split('/')[0]
        return f"{schema}/{concept_id}"

    @staticmethod
    def import_class(name):
        return locate(name.__module__ + "." + name.__name__)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['openfabric_pysdk']

package_data = \
{'': ['*']}

install_requires = \
['Flask-RESTful>=0.3.9,<0.4.0',
 'Flask>=1.1.2,<1.2.0',
 'apispec>=4.4.0,<4.5.0',
 'connexion>=2.9.0,<3.0.0',
 'flask-apispec>=0.11.0,<0.12.0',
 'marshmallow>=3.11.1,<3.12.0']

setup_kwargs = {
    'name': 'openfabric-pysdk',
    'version': '0.1.2',
    'description': 'Openfabric Python SDK',
    'long_description': '',
    'author': 'Andrei Tara',
    'author_email': 'andrei@openfabric.ai',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://openfabric.ai',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import json


class PyJSON(object):
    def __init__(self, d=None):
        if d is not None:
            if type(d) is str:
                d = json.loads(d)

            self.from_dict(d)

    def from_dict(self, d):
        self.__dict__ = {}
        for key, value in d.items():
            if type(value) is dict:
                value = PyJSON(value)
            self.__dict__[key] = value

    def to_dict(self):
        d = {}
        for key, value in self.__dict__.items():
            if type(value) is PyJSON:
                value = value.to_dict()
            d[key] = value
        return d

    def __repr__(self):
        return str(self.to_dict())

    def __setitem__(self, key, value):
        self.__dict__[key] = value

    def __getitem__(self, key):
        return self.__dict__[key]
